/*
Trick Candles
Copyright (C) 2024  Jeffry Johnston

This file is part of Trick Candles.

Trick Candles is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Trick Candles is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Trick Candles.  If not, see <https://www.gnu.org/licenses/>.
*/

#pragma once

#include <stdint.h>
#include <gint/clock.h>
#include <gint/display.h>
#include <gint/gint.h>
#include <gint/gray.h>
#include <gint/hardware.h>
#include <gint/keyboard.h>
#include <gint/rtc.h>
#include <gint/timer.h>
#ifndef FX9860G_G3A
#include <gint/usb-ff-bulk.h>
#include <gint/drivers/t6k11.h>
#endif

#define COMMAND_NONE 100
#define COMMAND_START 101
#define COMMAND_OSMENU 102
#define COMMAND_LEFT 103
#define COMMAND_RIGHT 104

typedef struct {
	uint8_t height;
	uint8_t is_lit;
} candle_t;

typedef enum __attribute__((__packed__)) {
	STATE_NOT_STARTED,
	STATE_RUNNING,
	STATE_GAME_OVER
} game_state_t;

void display_gray_off(void);
void display_game(void);

void game_init(void);
void game_start(void);
game_state_t game_get_state(void);
const candle_t *game_get_candle(const int x, const int y);
uint32_t game_get_score(void);
void game_move_left(void);
void game_move_right(void);
int game_get_sel_column(void);
uint8_t game_run(void);

void kbd_init(void);
uint8_t kbd_game(void);
