/*
Trick Candles
Copyright (C) 2024  Jeffry Johnston

This file is part of Trick Candles.

Trick Candles is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Trick Candles is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Trick Candles.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "candles.h"

extern bopti_image_t img_candle1;
extern bopti_image_t img_candle2;
extern bopti_image_t img_candle3;
extern bopti_image_t img_candle4;
extern bopti_image_t img_candle5;
extern bopti_image_t img_candle6;
extern bopti_image_t img_candle7;
extern bopti_image_t img_flame;
extern bopti_image_t img_gameover;
extern bopti_image_t img_help;
extern bopti_image_t img_melted;
extern bopti_image_t img_score;
extern bopti_image_t img_sel_candle1;
extern bopti_image_t img_sel_candle2;
extern bopti_image_t img_sel_candle3;
extern bopti_image_t img_sel_candle4;
extern bopti_image_t img_sel_candle5;
extern bopti_image_t img_sel_candle6;
extern bopti_image_t img_sel_candle7;
extern bopti_image_t img_wick;
extern font_t font_birthday;

void display_gray_off(void)
{
	dgray(DGRAY_OFF);
	dfont(NULL);
	dclear(C_WHITE);
	dprint(25, 28, C_BLACK, "Press any key");
}

static void init_gray(void)
{
	dfont(&font_birthday);
	dgray(DGRAY_ON);
}

static const int candle_height[] = {3, 5, 7, 10, 13, 16, 19, 22};
static const bopti_image_t *candle_image[] = {
	&img_melted,
	&img_candle1,
	&img_candle2,
	&img_candle3,
	&img_candle4,
	&img_candle5,
	&img_candle6,
	&img_candle7
};
static const bopti_image_t *sel_candle_image[] = {
	&img_melted,
	&img_sel_candle1,
	&img_sel_candle2,
	&img_sel_candle3,
	&img_sel_candle4,
	&img_sel_candle5,
	&img_sel_candle6,
	&img_sel_candle7
};

static void draw_score(void)
{
	dimage(59, 0, &img_score);
	uint32_t score = game_get_score();
	if (score > 999999)
		score = 999999;
	int x;
	if (score < 10)
		x = 103;
	else if (score < 100)
		x = 99;
	else if (score < 1000)
		x = 95;
	else if (score < 10000)
		x = 91;
	else if (score < 100000)
		x = 87;
	else
		x = 83;
	dprint(x, 0, C_BLACK, "%u", game_get_score());
}

static void draw_candles(void)
{
	const int sel_column = game_get_sel_column();
	for (int y = 0; y < 4; ++y)
		for (int x = 0; x < 5; ++x) {
			const candle_t *candle = game_get_candle(x, y);

			// Draw candlestick
			dimage(25*x + 6*y + 3 - (candle->height == 0),
				6*y + 40 - candle_height[candle->height],
				(x == sel_column) ? sel_candle_image[candle->height] :
					candle_image[candle->height]);

			// Draw flame or wick
			if (candle->height) {
				if (candle->is_lit)
					dimage(25*x + 6*y + 2,
						6*y + 28 - candle_height[candle->height],
						&img_flame);
				else
					dimage(25*x + 6*y + 4, 6*y + 35 - candle_height[candle->height], &img_wick);
			}
		}
}

void display_game(void)
{
	// Prepare gray engine
	init_gray();
	dclear(C_WHITE);

	switch (game_get_state()) {
	case STATE_NOT_STARTED:
		// Draw help
		dimage(0, 0, &img_help);
		break;
	case STATE_GAME_OVER:
		// Draw game over
		dimage(13, 0, &img_gameover);
	// Fall through
	case STATE_RUNNING:
		draw_score();
		break;
	}

	// Draw candles
	draw_candles();

	// Draw VRAM to display
	dupdate();
}
