/*
Trick Candles
Copyright (C) 2024  Jeffry Johnston

This file is part of Trick Candles.

Trick Candles is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Trick Candles is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Trick Candles.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "candles.h"

static uint32_t rng_state;
static candle_t candles[20];
static game_state_t state;
static uint32_t score;
static uint32_t score_tick;
static uint32_t candle_tick;
static uint16_t relighting_chance;
static int sel_column;

static uint32_t xorshift32(void)
{
    while (rng_state == 0)
        rng_state = rtc_ticks();

    uint32_t x = rng_state;
    x ^= x << 13;
    x ^= x >> 17;
    x ^= x << 5;
    rng_state = x;
    return x;
}

// Returns a random integer x, where 0<=x<max
int randint(const uint32_t max)
{
    if (max <= 1)
        return 0;

    uint32_t mask = max - 1;
    mask |= mask >> 1;
    mask |= mask >> 2;
    mask |= mask >> 4;
    mask |= mask >> 8;
    mask |= mask >> 16;

    uint32_t v;
    do {
        v = xorshift32() & mask;
    } while (v >= max);
    return v;
}

void game_init(void)
{
	for (int i = 0; i < 20; ++i) {
		candles[i].height = 7;
		candles[i].is_lit = 1;
	}
	state = STATE_NOT_STARTED;
	score = 0;
	score_tick = 0;
	candle_tick = 0;
	relighting_chance = 500;
	sel_column = -1;
}

void game_start(void)
{
	if (state != STATE_RUNNING) {
		game_init();
		sel_column = 0;
		state = STATE_RUNNING;
	}
}

game_state_t game_get_state(void)
{
	return state;
}

const candle_t *game_get_candle(const int x, const int y)
{
	return &(candles[5*y + x]);
}

uint32_t game_get_score(void)
{
	return score;
}

static void game_extinguish_candles(const int sel_column)
{
	if (state == STATE_RUNNING) {
		candles[sel_column + 0].is_lit = 0;
		candles[sel_column + 5].is_lit = 0;
		candles[sel_column + 10].is_lit = 0;
		candles[sel_column + 15].is_lit = 0;
	}
}

void game_move_left(void)
{
	if (sel_column > 0)
		game_extinguish_candles(--sel_column);
}

void game_move_right(void)
{
	if (sel_column < 4)
		game_extinguish_candles(++sel_column);
}

int game_get_sel_column(void)
{
	return sel_column;
}

uint8_t game_run(void)
{
	if (state != STATE_RUNNING)
		return 0;

	uint8_t redraw = 0;
	if (++score_tick > 4) {
		score_tick = 0;
		++score;
		redraw = 1;
	}

	// Burn lit candles
	if (++candle_tick > 80) {
		candle_tick = 0;
		for (int i = 0; i < 20; ++i)
			if (candles[i].is_lit && --(candles[i].height) == 0) {
				state = STATE_GAME_OVER;
				sel_column = -1;
			}
		if (relighting_chance > 100)
			relighting_chance -= 50;
	}
	if (state != STATE_RUNNING)
		return 1;

	// Randomly re-light candles
	for (int i = 0; i < 20; ++i)
		if (!candles[i].is_lit && !randint(relighting_chance))
			candles[i].is_lit = 1;

	return redraw;
}
