/*
Trick Candles
Copyright (C) 2024  Jeffry Johnston

This file is part of Trick Candles.

Trick Candles is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Trick Candles is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Trick Candles.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "candles.h"

#define KEY_NONE -1

void kbd_init(void)
{
#ifndef FX9860G_G3A
	usb_interface_t const *interfaces[] = { &usb_ff_bulk, NULL };
	usb_open(interfaces, GINT_CALL_NULL);
#endif
}

#ifndef FX9860G_G3A
/*
How to take an in-game screenshot:
1. Run: echo "SUBSYSTEM==\"usb\", MODE=\"0666\"" | sudo tee /etc/udev/rules.d/usb.rules
2. Run: fxlink -iw
3. Connect the calculator via USB then press EXIT (don't press F1 - F3)
4. Press "log" in the game
*/
static void take_screenshot(void)
{
	if (usb_is_open())
		usb_fxlink_screenshot_gray(1);
}
#endif

static unsigned int kbd_getkey(void)
{
	volatile int timeout = 1;
	key_event_t event = getkey_opt(GETKEY_BACKLIGHT, &timeout);
	if (event.type == KEYEV_NONE)
		return KEY_NONE;
	while (getkey_opt(GETKEY_BACKLIGHT, &timeout).type != KEYEV_NONE);

	uint key = event.key;

#ifndef FX9860G_G3A
	switch (key) {
	case KEY_LOG:
		take_screenshot();
		break;
	case KEY_OPTN:
		if (!isSlim())
			t6k11_backlight(-1);
		break;
	}
#endif
	return key;
}

uint8_t kbd_game(void)
{
	switch (kbd_getkey()) {
	case KEY_SHIFT:
		return COMMAND_START;
	case KEY_MENU:
		return COMMAND_OSMENU;
	case KEY_LEFT:
		return COMMAND_LEFT;
	case KEY_RIGHT:
		return COMMAND_RIGHT;
	case KEY_NONE:
	default:
		return COMMAND_NONE;
	}
}
