/*
Trick Candles
Copyright (C) 2024  Jeffry Johnston

This file is part of Trick Candles.

Trick Candles is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Trick Candles is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Trick Candles.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "candles.h"

static int handle_timer(volatile int *tick)
{
	*tick = 1;
	return TIMER_CONTINUE;
}

static int start_timer(volatile int *tick)
{
	const int timer = timer_configure(TIMER_ANY, 25000,
						GINT_CALL(handle_timer, tick));
	timer_start(timer);
	return timer;
}

int main(void)
{
	// Initialize game
	kbd_init();
	game_init();
	display_gray_off();
	volatile int tick = 1;
	int timer = start_timer(&tick);
	uint8_t redraw = 1;

	// Game loop
	while (1) {
		while (!tick)
			sleep();
		tick = 0;

		// Respond to player input
		const uint8_t input = kbd_game();
		switch (input) {
		case COMMAND_OSMENU:
			// Main menu
			timer_stop(timer);
			display_gray_off();
			gint_osmenu();
#ifdef FX9860G_G3A
            dupdate();
#endif
			game_init();
			tick = 1;
			timer = start_timer(&tick);
			redraw = 1;
			break;
		case COMMAND_START:
			game_start();
			redraw = 1;
			break;
		case COMMAND_NONE:
			redraw |= game_run();
			break;
		case COMMAND_LEFT:
			game_move_left();
			break;
		case COMMAND_RIGHT:
			game_move_right();
			break;
		default:
			break;
		}

		if (redraw) {
			// Display game state
			display_game();
			redraw = 0;
		}
	}

	return 1;
}
